-------------------------------------------------------------------------------
----------          CREACI DE TAULES DE LA BASE DE DADES            ----------
-------------------------------------------------------------------------------

--Creaci i inicialitzaci de seqncies.
CREATE SEQUENCE id_trans;   --Seqncia utilitzada en la taula de transaccions
CREATE SEQUENCE id_log;     --Seqncia emprada en la taula de log

--Creaci de la taula de log dels procediments
CREATE TABLE Log_proc(
  id_taula NUMBER,               --Automtic
  data_log DATE DEFAULT SYSDATE, --Data de la crida
  procediment VARCHAR2(50),       --Nom del procediment
  sor1 VARCHAR2(200),             --Variable de sortida
  ent_text VARCHAR2(200),         --Variable d'entrada reservada per textos llargs
  ent1 VARCHAR2(50),              --Variables d'entrada
  ent2 VARCHAR2(50),
  ent3 VARCHAR2(50),
  ent4 VARCHAR2(50),
  ent5 VARCHAR2(50),
  ent6 VARCHAR2(50),
  ent7 VARCHAR2(50),
  ent8 VARCHAR2(50),
  ent9 VARCHAR2(50),
  CONSTRAINT Clau_Log_proc
    PRIMARY KEY (id_taula)
);

-- Taula Proveidor. Representa els provedors de regals.
CREATE TABLE Proveidor(
  nom VARCHAR2(50),
  CONSTRAINT Clau_Proveidor
    PRIMARY KEY (nom)
);

--Taula IVA. Defineix els tipus d'IVA vlids.
CREATE TABLE IVA(
  tipus SMALLINT,
  CONSTRAINT Clau_IVA
    PRIMARY KEY (tipus),
  CONSTRAINT IVA_Correcte
    CHECK (tipus <= 100 AND tipus >= 0)
);

--Taula Categoria. Defineix les categories pertinents dels regals.
CREATE TABLE Categoria(
  nom VARCHAR2(30),
  CONSTRAINT Clau_Categoria
    PRIMARY KEY (nom)
);

--Taula Producte. Representa els possibles productes per a regalar.
CREATE TABLE Producte(
  nom VARCHAR2(40),
  proveidor VARCHAR2(50) NOT NULL,
  categoria VARCHAR2(30) NOT NULL,
  preu DECIMAL(10,2) NOT NULL,
  impost SMALLINT NOT NULL,
  descripcio VARCHAR2(200),
  imatge BLOB,
  CONSTRAINT Clau_Producte
    PRIMARY KEY (nom, proveidor),
  CONSTRAINT Proveidor_del_producte
    FOREIGN KEY (proveidor)
    REFERENCES Proveidor (nom),
  CONSTRAINT Categoria_del_producte
    FOREIGN KEY (categoria)
    REFERENCES Categoria (nom),
  CONSTRAINT Preu_Correcte        --Un preu no pot ser negatiu
    CHECK (preu >= 0),
  CONSTRAINT Impost_del_producte
    FOREIGN KEY (impost)
    REFERENCES IVA (tipus)
);

--Taula Usuari. Emmagatzema els correus electrnics de tots els usuaris.
CREATE TABLE Usuari(
  correu VARCHAR2(50),
  CONSTRAINT Clau_Usuari
    PRIMARY KEY (correu)
);

--Taula CP_id. Ajuda en la distribuci de codis postals per ciutats / provncies.
CREATE TABLE CP_id(
  id VARCHAR2(2),
  ciutat VARCHAR2(30),
  provincia VARCHAR2 (30),
  CONSTRAINT Clau_CP_id
    PRIMARY KEY (id)
);

--Taula Grup. Emmagatzema informaci sobre els grups de regals del sistema.
--S'identifica pel nom del grup i l'usuari homenatjat. D'aquesta manera, dos
--grups es poden dir igual si l'usuari homenatjat s diferent.
CREATE TABLE Grup(
  titol VARCHAR2(50),
  homenatjat VARCHAR2(50),
  dedicatoria VARCHAR2(500),
  data_fons DATE NOT NULL,
  data_regals DATE NOT NULL,
  privacitat VARCHAR2(1),
  receptor VARCHAR2(50) NOT NULL,
  adreca VARCHAR2(100) NOT NULL,
  poblacio VARCHAR2(50) NOT NULL,
  cp VARCHAR2(5) NOT NULL,
  cp_id VARCHAR2(2),
  finalitzat VARCHAR2(1) DEFAULT 'F',
  CONSTRAINT Clau_Homenatge
    PRIMARY KEY (titol, homenatjat),
  CONSTRAINT Homenatjat_del_producte
    FOREIGN KEY (homenatjat)
    REFERENCES USUARI (correu),
  CONSTRAINT Receptor_del_producte
    FOREIGN KEY (receptor)
    REFERENCES USUARI (correu),
  CONSTRAINT CP_del_receptor
    FOREIGN KEY (cp_id)
    REFERENCES CP_id (id),
  CONSTRAINT Privacitat_No_Boolea
    CHECK (privacitat = 'T' OR privacitat = 'F'),
  CONSTRAINT Finalitzat_No_Boolea
    CHECK (finalitzat = 'T' OR finalitzat = 'F'),
  CONSTRAINT Temps_suficient_entrega  --La empresa de missatgeria ha de tenir
    CHECK (data_regals > data_fons)   --temps per entregar els regals, 24h.
);

--Taula Estat_Desig. Defineix els possibles estats en que es pot trobar un desig.
CREATE TABLE Estat_Desig(
  estat VARCHAR2(20),
  CONSTRAINT Clau_Estat_Desig
    PRIMARY KEY (estat)
);

--Taula Desig. Defineix els desitjos dels usuaris, i si estn satisfets o no.
CREATE TABLE Desig(
  codi VARCHAR2(50),
  producte VARCHAR2(40),
  proveidor VARCHAR2(50),
  usuari VARCHAR2(50),
  data_pet DATE,            --Data de petici del desig
  data_estat DATE,          --Data de canvi d'estat del desig
  estat VARCHAR2(20),
  CONSTRAINT Clau_Desig
    PRIMARY KEY (codi),
  CONSTRAINT Producte_del_desig
    FOREIGN KEY (producte, proveidor)
    REFERENCES Producte (nom, proveidor),
  CONSTRAINT Usuari_del_desig
    FOREIGN KEY (usuari)
    REFERENCES Usuari (correu),
  CONSTRAINT Estat_del_desig
    FOREIGN KEY (estat)
    REFERENCES Estat_Desig (estat)
);

--Taula Estat_Regal. Defineix els possibles estats en que es pot trobar un regal.
CREATE TABLE Estat_Regal(
  estat VARCHAR2(20),
  CONSTRAINT Clau_Estat_Regal
    PRIMARY KEY (estat)
);

--Taula Regal. Emmagatzema informaci dels regals dels homenatges.
--Entenem un Regal com un Desig assignat a un Grup de participants.
CREATE TABLE Regal(
  desig VARCHAR2(50),
  titol_hom VARCHAR2(50),
  usuari_hom VARCHAR2(50),
  preu DECIMAL(10,2), --Automtica per motius d'eficincia. Preu amb impostos.
  estat VARCHAR2(20),
  CONSTRAINT Clau_Regal
    PRIMARY KEY (desig, titol_hom, usuari_hom),
  CONSTRAINT Producte_del_regal
    FOREIGN KEY (desig)
    REFERENCES Desig(codi),
  CONSTRAINT Grup_del_Regal
    FOREIGN KEY (titol_hom, usuari_hom)
    REFERENCES Grup (titol, homenatjat),
  CONSTRAINT Preu_regal_correcte      --Un preu no pot ser negatiu
    CHECK (preu >= 0),
  CONSTRAINT Estat_del_Regal
    FOREIGN KEY (estat)
    REFERENCES Estat_Regal (estat)
);

--Taula Integrant. Cont les dades de pertinena d'un usuari a un grup.
CREATE TABLE Integrant(
  titol_grup VARCHAR2(50),
  hom_grup VARCHAR2(50),
  usu_grup VARCHAR2(50),
  dedicatoria VARCHAR2(500),
  CONSTRAINT Clau_Integrant
    PRIMARY KEY (titol_grup, hom_grup, usu_grup),
  CONSTRAINT Grup_del_integrant 
    FOREIGN KEY (titol_grup, hom_grup)
    REFERENCES Grup (titol, homenatjat),
  CONSTRAINT Membre_del_grup
    FOREIGN KEY (usu_grup)
    REFERENCES Usuari (correu)
);

--Taula Participacio. Emmagatzema les participacions en el regals.
CREATE TABLE Participacio(
  desig VARCHAR2(50),
  titol_hom VARCHAR2(50),
  usuari_hom VARCHAR2(50),
  participant VARCHAR2(50),
  aportacio DECIMAL(10,2),
  CONSTRAINT Clau_Participacio
    PRIMARY KEY (desig, titol_hom, usuari_hom, participant),
  CONSTRAINT Regal_de_la_participacio 
    FOREIGN KEY (desig, titol_hom, usuari_hom)
    REFERENCES Regal (desig, titol_hom, usuari_hom),
  CONSTRAINT Implicat_en_la_participacio
    FOREIGN KEY (participant)
    REFERENCES Usuari (correu),
  CONSTRAINT Participacio_Valida CHECK (aportacio > 0)
);

--Taula Mode_Pagament. Optimitza els modes de pagament de les transaccions.
CREATE TABLE Mode_Pagament(
  modeP VARCHAR2(30),
  CONSTRAINT Clau_ModePagament
    PRIMARY KEY (modeP)
);

--Taula Tipus_Transaccio. Defineix els conceptes de "pagament" i "devoluci".
CREATE TABLE Tipus_Transaccio(
  tipus VARCHAR2(10),
  CONSTRAINT Clau_TipusTransaccio
    PRIMARY KEY (tipus)
);

--Taula Transaccio. Emmagatzema les transaccions del sistema.
CREATE TABLE Transaccio(
  id_tr NUMBER,
  instant DATE DEFAULT SYSDATE,
  tipus VARCHAR2(10),
  participant VARCHAR2(50),
  titol_hom VARCHAR2(50),
  usuari_hom VARCHAR2(50),
  regal VARCHAR2(50),
  dest_regal VARCHAR2(50),
  mode_p VARCHAR2(30),
  dades_p VARCHAR2(50),
  quantitat DECIMAL(10,2),
  anulada VARCHAR2(1) DEFAULT 'F',
  CONSTRAINT Clau_Transaccio
    PRIMARY KEY (id_tr),
  CONSTRAINT Tipus_de_transaccio FOREIGN KEY (tipus) REFERENCES Tipus_Transaccio (tipus),
  CONSTRAINT Participacio_de_la_transaccio
    FOREIGN KEY (regal, titol_hom, usuari_hom, participant)
    REFERENCES Participacio (desig, titol_hom, usuari_hom, participant),
  CONSTRAINT Transaccio_Valida
    CHECK ((tipus = 'Pagament' AND quantitat > 0) 
      OR  (tipus = 'Devoluci' AND quantitat < 0)),
  CONSTRAINT Estat_Transaccio_Valid
    CHECK (anulada = 'T' OR anulada = 'F')
);

-------------------------------------------------------------------------------
----------          CREACI DE TAULES DEL MDUL ESTADSTIC           ----------
-------------------------------------------------------------------------------

--R7.1
CREATE TABLE Regal_Vegades(
  producte VARCHAR2(40),                    --Producte que es vol consultar
  any_reg SMALLINT,                         --Any que es vol consultar
  vegades INT DEFAULT 0,                    --Cops que s'ha regalat un producte (7.1)
  CONSTRAINT Clau_Regal_Vegades
    PRIMARY KEY (producte, any_reg)
);

--R7.2, R7.3, R7.4, R7.5
CREATE TABLE Regal_Any(
  any_reg SMALLINT,                         --Any que es vol consultar
  adquirits INT DEFAULT 0,                  --Regals adquirits (7.2)
  imp_acumulat DECIMAL (10,2) DEFAULT 0.0,  --Import total regals (7.3)
  aux_aconseguits INT DEFAULT 0,            --Reg. aconseguits (per eficincia)
  aux_fallits INT DEFAULT 0,                --Reg. fallits (per eficincia)
  perc_aconseguits DECIMAL (10,2) DEFAULT 0.0, --%aconseguits/fallits (7.4)
  aux_despesa DECIMAL (10,2) DEFAULT 0.0,   --Despesa actual al sistema (per eficincia)
  mitja_despesa DECIMAL (10,4) DEFAULT 0.0, --Mitja de despesa per usuari (7.5)
  CONSTRAINT Clau_Regal_Any
    PRIMARY KEY (any_reg)
);

--R7.6, R7.7, R7.8, R7.9
CREATE TABLE Regal_Any_Mes(
  any_reg SMALLINT,                         --Any que es vol consultar
  mes SMALLINT,                             --Mes que es vol consultar
  aux_max_regals INT DEFAULT 0,             --Max regals aconseguits (per ef.)
  usuari_mes_regals VARCHAR2(50),           --Usuari amb ms regals (7.6)
  aux_max_despesa_usr DECIMAL (10,2) DEFAULT 0, --Max despesa d'usuari (per ef.)
  usuari_mes_despesa VARCHAR2(50),          --Usuari amb ms despesa (7.7)
  aux_max_participacions INT DEFAULT 0,     --Max particip. en regals (per ef.)
  usuari_mes_participacions VARCHAR2(50),   --Usuari amb ms participacions (7.8)
  aux_max_despesa_grup DECIMAL (10,2) DEFAULT 0, --Max despesa de grup (per ef.)
  nom_grup_mes_despesa VARCHAR2(50),        --Grup amb major despesa (7.9)
  hom_grup_mes_despesa VARCHAR2(50),        --Grup amb major despesa (7.9)
  CONSTRAINT Clau_Regal_Any_Mes
    PRIMARY KEY (any_reg, mes),
  CONSTRAINT Est_usu_mes_regals
    FOREIGN KEY (usuari_mes_regals)
    REFERENCES Usuari (correu),
  CONSTRAINT Est_usu_mes_despesa
    FOREIGN KEY (usuari_mes_despesa)
    REFERENCES Usuari (correu),
  CONSTRAINT Est_usu_mes_participacions
    FOREIGN KEY (usuari_mes_participacions)
    REFERENCES Usuari (correu)  
);